%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  Implementation of a inverse model based MOEA (IM-MOEA)
%%
%%  See the details of IM-MOEA in the following paper
%%
%%  R. Cheng, Y. Jin, K. Narukawa and B. Sendhoff, 
%%  A Multiobjective Evolutionary Algorithm using Gaussian Process based Inverse Modeling,
%%  IEEE Transactions on Evolutionary Computation, 2015
%%
%%  The source code IM-MOEA is implemented by Ran Cheng 
%%
%%  If you have any questions about the code, please contact: 
%%  Ran Cheng at r.cheng@surrey.ac.uk 
%%  Prof. Yaochu Jin at yaochu.jin@surrey.ac.uk
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


clear all;
addpath(genpath(pwd));

% F1 to F10 are modifed ZDT and DTLZ problems used in the TEC paper
problems = {'F1','F2','F3','F4','F5','F6','F7','F8','F9','F10'};

runnum = 1;         %number of runs
maxgen = 1000;      %maximal generation number
popsize = 150;      %population size
K = 15;             %number of reference vectors
L = 3;              %random group size

for probIdx = 2:2
    %test function initialization
    problem = problems{probIdx};
    [M n lower upper] = F_funcinit(probIdx);
    
    for run = 1 : runnum
        
        %IM-MOEA initialization
        univec = F_univec(K, M);       %uniform reference vectors
        K = size(univec, 1);           %number of reference vectors
        subpopsize = floor(popsize/K); %sub-population size
        
        rand('seed', sum(100 * clock));
        pop = repmat(lower, [popsize 1]) + rand(popsize,n).*(repmat(upper - lower, [popsize 1]));
        gen = 0;
        finalgen = 0;
        fitness = Bench_Func('value',pop, problem);
        partition = F_partition(fitness, univec); %population partition
        
        tic;
        %main loop
        while gen <= maxgen
            newpop = []; newfitness = [];
            %perform selection and reproduction in each subpopulation
            for k = 1 : K
                select = partition(k).c;
                if(~isempty(select))
                    subpop = pop(select,:);
                    subfitness = fitness(select,:);
                    
                    %elitism selection
                    if(size(subpop,1) > subpopsize )
                        %nondominated sorting using ENS-SS
                        [FrontValue,MaxFront] = P_sort(subfitness,'fixed',subpopsize);
                        Next = zeros(1,subpopsize);
                        NoN = numel(FrontValue,FrontValue<MaxFront);
                        Next(1:NoN) = find(FrontValue<MaxFront);
                        
                        %crowd distance sroting
                        Last = find(FrontValue==MaxFront);
                        CrowdDistance = P_distance(subfitness,FrontValue);
                        [~,Rank] = sort(CrowdDistance(Last),'descend');
                        Next(NoN+1:subpopsize) = Last(Rank(1:subpopsize-NoN));
                        
                        %selected population
                        subpop = subpop(Next,:);
                        subfitness = subfitness(Next,:);
                        
                    end;
                    %reserve the current elites
                    newpop = [newpop; subpop;];
                    newfitness = [newfitness; subfitness;];
                    %GP based reproduction
                    [offspring] =  F_reproduction(subpop, subfitness, finalgen, lower, upper, L);
                    newpop = [newpop; offspring];
                    newfitness = [newfitness;  Bench_Func('value',offspring, problem);];
                end
                
            end;
            
            %population update and partition
            pop =  newpop; fitness = newfitness;
            partition = F_partition(fitness, univec);
            
            %check if next is the final generation
            if gen == maxgen - 1
                finalgen = 1;
            end;
            
            %solution plot, which can be commented
            %F_plot(fitness, 'ro'); 
            
            gen = gen + 1
        end;
        %main loop finished
        toc

        %remove non-dominated solutions
        N = popsize;
        [FrontValue,MaxFront] = P_sort(fitness,'first');
        Next = find(FrontValue == 1);
        pop = pop(Next,:);
        fitness = fitness(Next,:);
        
        %solution plot, which can be commented
        F_plot(fitness, 'ro'); 
    end;
    
end;





